@@ -1,6 +1,6 @@
 /* Functions for handling the application structure.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <fcntl.h>
+#include "config.h"
 
 #include "process.h"
 #include "app.h"
@@ -60,6 +62,9 @@
   }
 #endif
 
+#define FTP_PORT 21
+#define FTP_QUIT(s) { ftp_send_cmd (s, "QUIT"); g_socket_close (s,NULL); }
+
 /* Initializes an app structure.
  *  input:
  *    boolean - the boolean flag from the cmd line args.
@@ -78,17 +83,25 @@ init_app (int boolean, int verbose)
 
   int ret_chk;
 
-  // Might change this to do the following:
-  // 1. Load default configuration
-  // 2. CHECK user configuration file.
-  // 3.  If it checks out, load it.
-  // 4.  Otherwise, don't do anything with it.
+  // Initialize the fonts and background colors.
 
-  ret_chk = the_app_read_config_file (app);
+  int i;
+  for (i = 0; i < NUM_FONT_TYPES; i++)
+    app->fonts[i] = NULL;
+  for (i = 0; i < NUM_BG_COLORS; i++)
+    app->bg_colors[i] = NULL;
+
+  /* First, read the default configuration.
+   * This way, if there are any problems reading the user's
+   *   configuration, then there will already be one in place.
+   */
+
+  ret_chk = the_app_read_default_config (app);
   if (ret_chk == -1)
     return NULL;
 
-  if (ret_chk == -2)
+  ret_chk = the_app_read_config_file (app);
+  if (ret_chk == -1)
     return NULL;
 
   app->icon = gdk_pixbuf_new_from_xpm_data (aris_icon_xpm);
@@ -126,10 +139,8 @@ init_app (int boolean, int verbose)
 int
 the_app_init_conn_pixbufs (aris_app * app)
 {
-  int i;
-
-  INIT_CONN_PIXBUF (0, and_conn_large_xpm, AND);
-  INIT_CONN_PIXBUF (1, or_conn_large_xpm, OR);
+  INIT_CONN_PIXBUF (0, and_conn_xpm, AND);
+  INIT_CONN_PIXBUF (1, or_conn_xpm, OR);
   INIT_CONN_PIXBUF (2, not_conn_xpm, NOT);
   INIT_CONN_PIXBUF (3, con_conn_xpm, CON);
   INIT_CONN_PIXBUF (4, bic_conn_xpm, BIC);
@@ -152,7 +163,7 @@ the_app_init_conn_pixbufs (aris_app * app)
 GdkPixbuf *
 the_app_get_conn_by_type (char * type)
 {
-  GdkPixbuf * ret;
+  GdkPixbuf * ret = NULL;
 
   if (!strcmp (type, AND))
     ret = the_app->conn_pixbufs[AND_CONN];
@@ -202,24 +213,15 @@ int
 the_app_get_color_by_type (aris_app * app, char * type)
 {
   int ret;
-
-  if (!strcmp (type, _("Default")))
-      ret = BG_COLOR_DEFAULT;
-
-  if (!strcmp (type, _("Conclusion")))
-      ret = BG_COLOR_CONC;
-
-  if (!strcmp (type, _("Reference")))
-    ret = BG_COLOR_REF;
-
-  if (!strcmp (type, _("Good")))
-    ret = BG_COLOR_GOOD;
-
-  if (!strcmp (type, _("Bad")))
-    ret = BG_COLOR_BAD;
-
-  if (!strcmp (type, _("Selection")))
-    ret = BG_COLOR_SEL;
+  int i;
+  for (i = 4; i < NUM_DISPLAY_CONFS; i++)
+    {
+      if (!strcmp (type, display_conf[i].label))
+        {
+          ret = display_conf[i].id;
+          break;
+        }
+    }
 
   return ret;
 }
@@ -235,33 +237,16 @@ the_app_get_color_by_type (aris_app * app, char * type)
 char *
 the_app_get_color_by_index (aris_app * app, int index)
 {
-  char * ret;
+  char * ret = NULL;
+  int i;
 
-  switch (index)
+  for (i = 4; i < NUM_DISPLAY_CONFS; i++)
     {
-    case BG_COLOR_DEFAULT:
-      ret = _("Default");
-      break;
-
-    case BG_COLOR_CONC:
-      ret = _("Conclusion");
-      break;
-
-    case BG_COLOR_REF:
-      ret = _("Reference");
-      break;
-
-    case BG_COLOR_BAD:
-      ret = _("Bad");
-      break;
-
-    case BG_COLOR_GOOD:
-      ret = _("Good");
-      break;
-
-    case BG_COLOR_SEL:
-      ret = _("Select");
-      break;
+      if (display_conf[i].id == index)
+        {
+          ret = display_conf[i].label;
+          break;
+        }
     }
 
   return ret;
@@ -277,23 +262,41 @@ the_app_get_color_by_index (aris_app * app, int index)
 int
 the_app_get_font_by_name (aris_app * app, char * name)
 {
-  int ret;
+  int ret = -1;
+  int i;
 
-  if (!strcmp (name, _("Small")))
-    ret = FONT_TYPE_SMALL;
-
-  if (!strcmp (name, _("Medium")))
-    ret = FONT_TYPE_MEDIUM;
-
-  if (!strcmp (name, _("Large")))
-    ret = FONT_TYPE_LARGE;
-
-  if (!strcmp (name, _("Default")))
-    ret = FONT_TYPE_CUSTOM;
+  for (i = 0; i < 4; i++)
+    {
+      if (!strcmp (name, display_conf[i].label))
+        {
+          ret = display_conf[i].id;
+          break;
+        }
+    }
 
   return ret;
 }
 
+/* Read the default configuration.
+ *  input:
+ *    app - the main application structure.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
+int
+the_app_read_default_config (aris_app * app)
+{
+  int ret_chk;
+
+  unsigned char * conf_def = config_default ();
+
+  ret_chk = conf_file_read (conf_def, app);
+  if (ret_chk == AEC_MEM)
+    return AEC_MEM;
+
+  return 0;
+}
+
 /* Constructs the config file path, then reads from it.
  *  input:
  *   none.
@@ -305,17 +308,24 @@ the_app_read_config_file (aris_app * app)
 {
   FILE * conf_file;
   char * path, * home_dir;
-  int alloc_size, exists, ret_chk;
+  int alloc_size, exists, ret_chk, path_pos = 0;
 
   home_dir = getenv ("HOME");
   if (!home_dir)
     home_dir = getenv ("HOMEPATH");
 
   alloc_size = strlen (home_dir) + strlen (CONF_FILE) + 1;
+#ifdef WIN32
+  alloc_size += strlen (getenv ("HOMEDRIVE")) + 1;
+#endif
   path = (char *) calloc (alloc_size + 1, sizeof (char));
-  CHECK_ALLOC (path, -1);
+  CHECK_ALLOC (path, AEC_MEM);
 
-  sprintf (path, "%s/%s", home_dir, CONF_FILE);
+#ifdef WIN32
+  path_pos += sprintf (path, "%s\\", getenv ("HOMEDRIVE"));
+#endif
+ 
+  sprintf (path + path_pos, "%s/%s", home_dir, CONF_FILE);
   exists = access (path, F_OK);
   if (exists == -1)
     {
@@ -327,38 +337,38 @@ the_app_read_config_file (aris_app * app)
   conf_file = fopen (path, "r+");
   if (!conf_file)
     {
-      perror (NULL);
+      perror ("load_config_file");
       return -2;
     }
+  free (path);
 
-  ret_chk = conf_file_read (conf_file, app);
-  if (ret_chk == -1)
-    return -1;
+  unsigned char * buffer;
+  size_t size;
+
+  GET_LEN (conf_file, size);
+  buffer = (unsigned char *) calloc (size + 1, sizeof (char));
+  CHECK_ALLOC (buffer, -1);
+
+  // Because Losedows hates it when we try to read the entire file at once.
+  int f_pos;
+  for (f_pos = 0; f_pos < size; f_pos+=512)
+    fread (buffer + f_pos, 1, 512, conf_file);
+  buffer[size] = '\0';
 
   fclose (conf_file);
 
+  ret_chk = conf_file_read (buffer, app);
+  if (ret_chk == -1)
+    return -1;
+
+  free (buffer);
+
   if (ret_chk == -2)
     {
-      ret_chk = the_app_make_default_config_file (path);
-      if (ret_chk == -1)
-	return -1;
-
-      conf_file = fopen (path, "r+");
-      if (!conf_file)
-	{
-	  perror (NULL);
-	  return -2;
-	}
-
-      ret_chk = conf_file_read (conf_file, app);
-      if (ret_chk == -1)
-	return -1;
-
-      fclose (conf_file);
+      // Maybe notify the user that there is a problem.
+      // Otherwise, it shouldn't matter.
     }
 
-  free (path);
-
   return 0;
 }
 
@@ -366,7 +376,7 @@ the_app_read_config_file (aris_app * app)
  *  input:
  *   path - the path to the configuration file.
  *  output:
- *   0 on success, -2 on read error.
+ *   0 on success, -2 on read error, -1 on memory error.
  */
 int
 the_app_make_default_config_file (char * path)
@@ -380,10 +390,21 @@ the_app_make_default_config_file (char * path)
       return -2;
     }
 
-  int len;
-  len = strlen (config_default);
+  unsigned char * conf_def;
+  conf_def = config_default ();
+  if (!conf_def)
+    return AEC_MEM;
+
+  int len, rc;
+  len = strlen (conf_def);
+
+  rc = fwrite (conf_def, 1, len, config_file);
+  if (rc != len)
+    {
+      perror ("default_config_file_fwrite");
+      return -2;
+    }
 
-  fwrite (config_default, 1, len, config_file);
   fclose (config_file);
 
   return 0;
@@ -399,14 +420,14 @@ int
 the_app_set_focus (aris_proof * ap)
 {
   if (!ap)
-    return -1;
+    return AEC_MEM;
 
   the_app->focused = ap;
 
   item_t * itm;
   itm = ls_find (the_app->guis, ap);
   if (!itm)
-    return -1;
+    return AEC_MEM;
 
   // This is already the first element,
   // so no need to re-order the elements.
@@ -419,8 +440,6 @@ the_app_set_focus (aris_proof * ap)
   else
     the_app->guis->tail = itm->prev;
 
-  item_t * h_itm = the_app->guis->head;
-
   itm->next = the_app->guis->head;
   itm->prev = NULL;
   the_app->guis->head->prev = itm;
@@ -450,15 +469,15 @@ the_app_add_gui (aris_proof * ap)
   item_t * itm;
   itm = ls_ins_obj (the_app->guis, ap, the_app->guis->tail);
   if (!itm)
-    return -1;
+    return AEC_MEM;
 
   gtk_window_set_icon (GTK_WINDOW (SEN_PARENT (ap)->window), the_app->icon);
-  gtk_window_set_icon (GTK_WINDOW (ap->goal->window), the_app->icon);
+  gtk_window_set_icon (GTK_WINDOW (SEN_PARENT (ap->goal)->window), the_app->icon);
 
   int ret;
   ret = the_app_set_focus (ap);
   if (ret < 0)
-    return -1;
+    return AEC_MEM;
 
   return 0;
 }
@@ -508,13 +527,15 @@ int
 app_quit ()
 {
   item_t * g_itr, * ng_itr;
+  int rc;
 
   for (g_itr = the_app->guis->head; g_itr; g_itr = ng_itr)
     {
       ng_itr = g_itr->next;
 
-      if (gui_destroy ((aris_proof *) g_itr->value) < 0)
-	return -1;
+      rc = gui_destroy ((aris_proof *) g_itr->value);
+      if (rc < 0)
+        return -1;
 
       g_itr->next = g_itr->prev = g_itr->value = NULL;
       free (g_itr);
@@ -532,8 +553,55 @@ app_quit ()
   return 0;
 }
 
-// Thanks to the developers of the inetutils package for this next part.
-// The ftp utility's source code was very helpful with this.
+/* Sets the color with index INDEX to the specified RGB values.
+ *  input:
+ *    app - the main application object.
+ *    index - the index of the color to setup.
+ *    red, green, blue - the RGB values of the new color ([0,255]).
+ *  output:
+ *    0 on success.
+ */
+int
+app_set_color (aris_app * app, int index, int red, int green, int blue)
+{
+  INIT_COLOR (app->bg_colors[index], red, green, blue);
+
+  if (app->guis)
+    {
+      item_t * gui_iter;
+      for (gui_iter = app->guis->head; gui_iter;
+	   gui_iter = gui_iter->next)
+	{
+	  aris_proof * ap = gui_iter->value;
+	  sen_parent_children_set_bg_color (SEN_PARENT (ap));
+	}
+    }
+
+  return 0;
+}
+
+/* Thanks to the developers of the inetutils package for this next part.
+ * The ftp utility's source code was very helpful with this.
+ */
+
+/* Send a command to an ftp server.
+ *  input:
+ *    ftp_socket - the socket that connects to the ftp server.
+ *    cmd - the command to send to the ftp server.
+ *  output:
+      none
+ */
+void
+ftp_send_cmd (GSocket * ftp_sock, const char * cmd)
+{
+  char * buf;
+  int len;
+
+  buf = (char *) calloc (strlen (cmd) + 3, sizeof (char));
+  len = sprintf (buf, "%s\r\n", cmd);
+  g_socket_send (ftp_sock, buf, len, NULL, NULL);
+  free (buf);
+}
 
 /* Get a response from the ftp server.
  *  input:
@@ -543,23 +611,25 @@ app_quit ()
  *    the response code.
  */
 int
-ftp_get_response (FILE * ftp_sock, int * port)
+ftp_get_response (GSocket * ftp_sock, int * port)
 {
-  char c;
   char response_buffer[2048], * buf_str;
-  int ret_code;
+  int ret_code, rc;
 
   buf_str = response_buffer;
 
-  while ((*buf_str = getc (ftp_sock)) != '\n')
+  while (1)
     {
-      if (*buf_str == EOF)
+      g_socket_receive (ftp_sock, buf_str, 1, NULL, NULL);
+      if (*buf_str == EOF || *buf_str == '\n')
 	break;
       buf_str++;
     }
 
   *buf_str = '\0';
-  sscanf (response_buffer, "%i", &ret_code);
+  rc = sscanf (response_buffer, "%i", &ret_code);
+  if (rc != 1)
+    return AEC_IO;
 
   if (port)
     {
@@ -576,82 +646,6 @@ ftp_get_response (FILE * ftp_sock, int * port)
   return ret_code;
 }
 
-#define FTP_PORT 21
-
-/* Initiates the ftp connection.
- *  input:
- *    ip_addr - the IP Address of the ftp server.
- *  output:
- *    A handle to the ftp server on success, NULL on error.
- */
-FILE *
-ftp_connect (char * ip_addr)
-{
-  int ret_chk;
-  FILE * ret;
-
-  GSocket * sock;
-
-  sock = g_socket_new (G_SOCKET_FAMILY_IPV4, G_SOCKET_TYPE_STREAM,
-		       G_SOCKET_PROTOCOL_DEFAULT, NULL);
-  if (!sock)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
-  GInetAddress * inet_addr;
-  GSocketAddress * addr;
-
-  inet_addr = g_inet_address_new_from_string (ip_addr);
-  addr = g_inet_socket_address_new (inet_addr, FTP_PORT);
-  if (g_socket_connect (sock, addr, NULL, NULL) == FALSE)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
-  ret = fdopen (g_socket_get_fd (sock), "w+");
-  if (!ret)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
-  ret_chk = ftp_get_response (ret, NULL);
-
-  if (ret_chk != 220)
-    {
-      fprintf (ret, "QUIT\r\n");
-      fclose (ret);
-      return NULL;
-    }
-
-  fprintf (ret, "USER anonymous\r\n");
-  fflush (ret);
-  ret_chk = ftp_get_response (ret, NULL);
-
-  if (ret_chk != 331)
-    {
-      fprintf (ret, "QUIT\r\n");
-      fclose (ret);
-      return NULL;
-    }
-
-  fprintf (ret, "PASS stuff\r\n");
-  fflush (ret);
-  ret_chk = ftp_get_response (ret, NULL);
-
-  if (ret_chk != 230)
-    {
-      fprintf (ret, "QUIT\r\n");
-      fclose (ret);
-      return NULL;
-    }
-
-  return ret;
-}
-
 /* Sends a file to the ftp server.
  *  input:
  *    ftp_sock - a handle to the ftp server.
@@ -661,86 +655,172 @@ ftp_connect (char * ip_addr)
  *    0 on success, -1 on memory error, -2 on file error.
  */
 int
-ftp_send (FILE * ftp_sock, char * file_name, char * buffer)
+ftp_send (GSocket * ftp_sock, char * file_name, char * buffer)
 {
   int port;
-  int len, c;
-  FILE * write_file;
   char * buf;
   int ret_chk;
 
   buf = buffer;
 
   GSocket * sock;
-
-  sock = g_socket_new (G_SOCKET_FAMILY_IPV4, G_SOCKET_TYPE_STREAM,
-		       G_SOCKET_PROTOCOL_DEFAULT, NULL);
+  sock = g_socket_new (G_SOCKET_FAMILY_IPV4,
+		       G_SOCKET_TYPE_STREAM,
+		       G_SOCKET_PROTOCOL_TCP,
+		       NULL);
   if (!sock)
     {
-      perror (NULL);
-      return -2;
+      return AEC_IO;
     }
 
-  fprintf (ftp_sock, "PASV\r\n");
-  fflush (ftp_sock);
+  ftp_send_cmd (ftp_sock, "PASV");
   ret_chk = ftp_get_response (ftp_sock, &port);
-  if (ret_chk != 221)
+  if (ret_chk != 227)
     {
-      fprintf (ftp_sock, "QUIT\r\n");
-      fclose (ftp_sock);
-      return -2;
+      FTP_QUIT (ftp_sock);
+      return AEC_IO;
     }
 
-  GInetAddress * host;
-  GSocketAddress * response;
-  host = g_inet_address_new_from_string (the_app->ip_addr);
-  response = g_inet_socket_address_new (host, port);
+  GSocketAddress * in_addr;
+  GInetAddress * addr;
+  addr = g_inet_address_new_from_string (the_app->ip_addr);
+  in_addr = g_inet_socket_address_new (addr, port);
 
-  if (g_socket_connect (sock, response, NULL, NULL) == FALSE)
+  gboolean ret_val;
+  ret_val = g_socket_connect (sock, in_addr, NULL, NULL);
+  if (!ret_val)
     {
-      perror (NULL);
-      return -2;
+      return AEC_IO;
     }
 
-  fprintf (ftp_sock, "STOR %s\r\n", file_name);
-  fflush (ftp_sock);
+  char * abuf;
+  size_t alen;
+
+  alen = strlen (file_name) + 5;
+  abuf = (char *) calloc (alen + 1, sizeof (char));
+  CHECK_ALLOC (abuf, -1);
+
+  sprintf (abuf, "STOR %s", file_name);
+  ftp_send_cmd (ftp_sock, abuf);
+  free (abuf);
   ret_chk = ftp_get_response (ftp_sock, NULL);
 
   if (ret_chk != 150)
     {
-      fprintf (ftp_sock, "QUIT\r\n");
-      fclose (ftp_sock);
-      return -2;
-    }
-
-  write_file = fdopen (g_socket_get_fd (sock), "w");
-  if (!write_file)
-    {
-      perror (NULL);
-      return -1;
+      FTP_QUIT (ftp_sock);
+      return AEC_IO;
     }
 
   while (*buf)
     {
       if (*buf == '\n')
-	putc ('\r', write_file);
-
-      putc (*buf, write_file);
+	g_socket_send (sock, "\r", 1, NULL, NULL);
+      g_socket_send (sock, buf, 1, NULL, NULL);
       buf++;
     }
 
-  fclose (write_file);
+  g_socket_close (sock, NULL);
+
   ret_chk = ftp_get_response (ftp_sock, NULL);
   if (ret_chk != 226)
     {
-      fprintf (ftp_sock, "QUIT\r\n");
-      fclose (ftp_sock);
-      return -2;
+      FTP_QUIT (ftp_sock);
+      return AEC_IO;
     }
 
   return 0;
 }
 
+/* Initiates the ftp connection.
+ *  input:
+ *    ip_addr - the IP Address of the ftp server.
+ *  output:
+ *    A handle to the ftp server on success, NULL on error.
+ */
+GSocket *
+ftp_connect (char * ip_addr)
+{
+  GSocket * ret;
+  ret = g_socket_new (G_SOCKET_FAMILY_IPV4,
+		      G_SOCKET_TYPE_STREAM,
+		      G_SOCKET_PROTOCOL_TCP,
+		      NULL);
+  if (!ret)
+    {
+      return NULL;
+    }
+
+  GSocketAddress * in_addr;
+  GInetAddress * addr;
+  addr = g_inet_address_new_from_string (ip_addr);
+  in_addr = g_inet_socket_address_new (addr, FTP_PORT);
+
+  // Set a timeout to prevent the socket from taking too long.
+  g_socket_set_timeout (ret, 5);
+  
+  gboolean ret_val;
+  ret_val = g_socket_connect (ret, in_addr, NULL, NULL);
+
+  if (!ret_val)
+    return NULL;
+
+  // I'm not sure what the response code is supposed to be here.
+  // I *think* 200
+  int rc, ret_chk;
+  rc = ftp_get_response (ret, NULL);
+  if (rc < 0)
+    {
+      FTP_QUIT (ret);
+      return NULL;
+    }
+
+  ftp_send_cmd (ret, "USER anonymous");
+  ret_chk = ftp_get_response (ret, NULL);
+
+  if (ret_chk != 331)
+    {
+      FTP_QUIT (ret);
+      return NULL;
+    }
+
+  char * pass_buf;
+
+  pass_buf = (char *) calloc (strlen (the_app->grade_pass) + 6,
+			      sizeof (char));
+  CHECK_ALLOC (pass_buf, NULL);
+
+  sprintf (pass_buf, "PASS %s", the_app->grade_pass);
+
+  ftp_send_cmd (ret, pass_buf);
+  free (pass_buf);
+  ret_chk = ftp_get_response (ret, NULL);
+
+  if (ret_chk != 230)
+    {
+      FTP_QUIT (ret);
+      return NULL;
+    }
+
+  char * dir_buf;
+  dir_buf = (char * ) calloc (strlen (the_app->grade_dir) + 6,
+			      sizeof (char));
+  CHECK_ALLOC (dir_buf, NULL);
+
+  sprintf (dir_buf, "CWD %s", the_app->grade_dir);
+
+  ftp_send_cmd (ret, dir_buf);
+  free (dir_buf);
+  ret_chk = ftp_get_response (ret, NULL);
+
+  if (ret_chk != 250)
+    {
+      FTP_QUIT (ret);
+      return NULL;
+    }
+
+  return ret;
+}
+
 /* Submits all open proofs for grading.
  *  input:
  *    user_email - the email address of the user.
@@ -754,14 +834,16 @@ the_app_submit (const char * user_email, const char * instr_email,
 		struct submit_ent * entries)
 {
   int ret_chk;
-  FILE * ftp_file;
+  GSocket * ftp_file;
 
   ftp_file = ftp_connect (the_app->ip_addr);
+  if (!ftp_file)
+    return -3;
 
   char * email_base;
 
   email_base = (char *) calloc (strlen (user_email), sizeof (char));
-  CHECK_ALLOC (email_base, -1);
+  CHECK_ALLOC (email_base, AEC_MEM);
 
   ret_chk = sscanf (user_email, "%[^@]@", email_base);
   if (ret_chk != 1)
@@ -773,20 +855,21 @@ the_app_submit (const char * user_email, const char * instr_email,
   char * dir_file_name, * dir_buffer;
 
   dir_file_name = (char *) calloc (strlen (email_base) + 11, sizeof (char));
-  CHECK_ALLOC (dir_file_name, -1);
+  CHECK_ALLOC (dir_file_name, AEC_MEM);
   sprintf (dir_file_name, "%s.directive", email_base);
 
   int dir_pos = 0;
 
-  dir_buffer = (char *) calloc (strlen (user_email) + strlen (instr_email)
-				+ 16, sizeof (char));
-  CHECK_ALLOC (dir_buffer, -1);
+  int alloc_size = strlen (user_email) + strlen (instr_email) + 16;
+
+  dir_buffer = (char *) calloc (alloc_size + 3, sizeof (char));
+  CHECK_ALLOC (dir_buffer, AEC_MEM);
 
   dir_pos += sprintf (dir_buffer, "user: %s\ninstr: %s\n",
 		      user_email, instr_email);
   int i;
 
-  for (i = 0; i < the_app->guis->num_stuff; i++)
+  for (i = 0; entries[i].hw; i++)
     {
       char * hw, * file_name;
 
@@ -799,15 +882,15 @@ the_app_submit (const char * user_email, const char * instr_email,
       base_name = g_file_get_basename (file);
 
       base = (char *) calloc (strlen (base_name), sizeof (char));
-      CHECK_ALLOC (base, -1);
+      CHECK_ALLOC (base, AEC_MEM);
 
       sscanf (base_name, "%[^.].tle", base);
-      g_free (base_name);
+      free (base_name);
 
       ap_file_name = (char *) calloc (strlen (email_base)
 				      + strlen (base) + 6,
 				      sizeof (char));
-      CHECK_ALLOC (ap_file_name, -1);
+      CHECK_ALLOC (ap_file_name, AEC_MEM);
       sprintf (ap_file_name, "%s-%s.tle", base, email_base);
 
       free (base);
@@ -821,21 +904,43 @@ the_app_submit (const char * user_email, const char * instr_email,
 	{
 	  free (ap_file_name);
 	  perror (NULL);
-	  return -2;
+	  return AEC_IO;
 	}
 
       GET_LEN (ap_file, len);
 
       ap_buffer = (char *) calloc (len + 1, sizeof (char));
-      CHECK_ALLOC (ap_buffer, -1);
+      CHECK_ALLOC (ap_buffer, AEC_MEM);
 
-      fread (ap_buffer, 1, len, ap_file);
+      ret_chk = fread (ap_buffer, 1, len, ap_file);
+      if (ret_chk != len)
+        {
+          free (ap_buffer);
+          free (dir_buffer);
+          free (email_base);
+          free (dir_file_name);
+          free (ap_file_name);
+          fclose (ap_file);
+          FTP_QUIT (ftp_file);
+          return AEC_IO;
+        }
       ap_buffer[len] = '\0';
       fclose (ap_file);
 
       ret_chk = ftp_send (ftp_file, ap_file_name, ap_buffer);
-      if (ret_chk == -1)
-	return -1;
+      if (ret_chk == AEC_MEM)
+	return AEC_MEM;
+
+      if (ret_chk == AEC_IO)
+        {
+          free (ap_buffer);
+          free (dir_buffer);
+          free (email_base);
+          free (dir_file_name);
+          free (ap_file_name);
+          FTP_QUIT (ftp_file);
+          return AEC_IO;
+        }
 
       free (ap_buffer);
 
@@ -854,18 +959,20 @@ the_app_submit (const char * user_email, const char * instr_email,
   free (email_base);
 
   ret_chk = ftp_send (ftp_file, dir_file_name, dir_buffer);
-  if (ret_chk == -1)
-    return -1;
+  if (ret_chk == AEC_MEM)
+    return AEC_MEM;
+  if (ret_chk == AEC_IO)
+    {
+      free (dir_file_name);
+      free (dir_buffer);
+      FTP_QUIT (ftp_file);
+      return AEC_IO;
+    }
 
   free (dir_file_name);
   free (dir_buffer);
 
-  fprintf (ftp_file, "QUIT\r\n");
-  fflush (ftp_file);
-
-  g_socket_shutdown (g_socket_new_from_fd (fileno (ftp_file), NULL),
-		     TRUE, TRUE, NULL);
-  fclose (ftp_file);
+  FTP_QUIT (ftp_file);
 
   return 0;
 }
